#!/bin/sh  
# -- BEGIN COPYRIGHT --
# IBM SecureWay Firewall Version 4.1
# Licensed Materials - Property of IBM
#
# 5697-F48
#
# (C) Copyright IBM Corp. 1985, 1994, 1999 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -- END COPYRIGHT --

#
# Program File: filter.sh
#

PRODUCT=SECUREWAY_FIREWALL

# Description: converts back slashed to forward slashes
#
# Usage: 
#        to_unix $TARGET_PATH TARGET_PATH
to_unix()
{
  if [ "$INTERP" = "w32-ix86" ] ; then
    UNIX_PATH=`echo $1 | sed -e 's|\\\\|/|g'`
  else
    UNIX_PATH=`printf "%s\n" "$1" | sed -e 's|\\\\|/|g'`
  fi
 
  eval $2=$UNIX_PATH
}

REG="reg2.exe"

# ----------------------------------------------------------------------
# Discover if we are on an LCF endpoint.
# ----------------------------------------------------------------------
if [ "x$LCF_BINDIR" != "x" ]; then
  EP=TRUE
  if [ "x$OS" = "xWindows_NT" ]; then
    ETC=$SystemRoot/system32/drivers/etc/Tivoli
    NULL=nul
    if [ -f $TOOLS/$REG ] ; then
      FW_DIR=`$REG -p "SOFTWARE/IBM/IBMFirewall/4.1" | grep RootDirectory | awk -F= '{print $2}'`
      to_unix $FW_DIR FW_DIR
    else
      echo Missing $REG binary.  Please install the plus module
      echo on the Firewall server.
      exit 1
    fi
  else
    ETC=/etc/Tivoli
    NULL=/dev/null
    LANG=en_US
    export LANG
  fi
  TEMP=/tmp
else
  EP=FALSE
fi

if [ x"$EP" = x"FALSE" ] ; then
  if [ -f $SystemRoot/system32/drivers/etc/Tivoli/setup_env.sh ] ; then
          . $SystemRoot/system32/drivers/etc/Tivoli/setup_env.sh
          if [ -f $BINDIR/../generic_unix/TME/PLUS/SECUREWAY_FIREWALL/$REG ] ; then
            FW_DIR=`$BINDIR/../generic_unix/TME/PLUS/SECUREWAY_FIREWALL/$REG -p "SOFTWARE/IBM/IBMFirewall/4.1" | grep RootDirectory | awk -F= '{print $2}'`
            to_unix $FW_DIR FW_DIR
            NLSPATH=$FW_DIR/msg/en_US/%N
            export NLSPATH
          else
            echo Missing $REG binary.  Please install the plus module
            echo on the Firewall server.
            exit 1
          fi
	  TEMP=$DBDIR/tmp
  else
          . /etc/Tivoli/setup_env.sh
	  TEMP=/tmp
          NLSPATH=$NLSPATH:/usr/lib/nls/msg/%L/%N:/usr/lib/nls/msg/%L/%N.cat:/usr/lib/nls/msg/prime/%N
          export NLSPATH
  fi
  OSERV=`objcall 0.0.0 get_oserv`
  INST_DIR=`objcall $OSERV query install_dir`
  PATH=$INST_DIR/generic_unix/TME/PLUS/SECUREWAY_FIREWALL:$BINDIR/AEF:$PATH
  export PATH
fi

#  Debugging code: by creating a file called /tmp/${PRODUCT}debug in
#     the appropriate TEMP directory, the output of scripts is
#     set to debugging (should appear in stderr for the task)

DEBUGFILE=$TEMP/${PRODUCT}debug

if [ -f $DEBUGFILE ]; then
  set -x
fi

# find out if we're starting or stopping the socks server
if [ x"$1" != x ] ; then
  # translate entry to all lowercase so we can
  # see if they've entered start or stop
  ACTION=`echo $1 | tr '[A-Z]' '[a-z]'`
  if [ x"$ACTION" != x"stop" -a x"$ACTION" != x"start" ] ; then
    echo "You need to specify either Start or Stop!"
    echo "Please rerun this task!"
    exit 1
  fi
else
  echo "You need to specify either Start or Stop!"
  echo "Please rerun this task!"
  exit 1
fi

#Must unset CHILD_OF_OSERV or it will end up in the firewall environment
#and could really hose up the CLI calls back to TME
unset CHILD_OF_OSERV

env

if [ x"$ACTION" = x"start" ] ; then
  case `uname -sr` in

	"AIX"*)
                if [ -f "/usr/bin/fwfilter" ] ; then 
                  /usr/bin/fwfilter cmd=update
                  if [ $? -eq 0 ] ; then
                    echo Started IBM Firewall Filter Engine!
                    exit 0
                  else
                    echo Error in starting IBM Firewall Filter Engine! >&2
                    exit 1
                  fi
                else
                  echo IBM Firewall Filter Engine binary not found! >&2
                  exit 1
                fi 
		;;

	"Windows_NT 4.0"*)
                if [ -f "$FW_DIR/bin/fwfilter.exe" ] ; then
                  $FW_DIR/bin/fwfilter cmd=update
                  if [ $? -eq 0 ] ; then
                    echo Started IBM Firewall Filter Engine!
                    exit 0
                  else
                    echo Error in starting IBM Firewall Filter Engine! >&2
                    exit 1
                  fi
                else
                  echo IBM Firewall Filter Engine binary not found! >&2
                  exit 1
                fi
	        ;;
  esac
else
  case `uname -sr` in

        "AIX"*)
               if [ -f "$FW_DIR/bin/fwfilter" ] ; then
                 /usr/bin/fwfilter cmd=shutdown
                 if [ $? -eq 0 ] ; then
                   echo Stopped IBM Firewall Filter Engine!
                   exit 0
                 else
                   echo Error in stopping IBM Firewall Filter Engine! >&2
                   exit 1
                 fi
               else
                 echo IBM Firewall Filter Engine binary not found! >&2
                 exit 1
               fi
               ;;

        "Windows_NT 4.0"*)
                if [ -f "$FW_DIR/bin/fwfilter.exe" ] ; then             
                  $FW_DIR/bin/fwfilter cmd=shutdown
                  if [ $? -eq 0 ] ; then
                    echo Stopped IBM Firewall Filter Engine!
                    exit 0
                  else
                    echo Error in stopping IBM Firewall Filter Engine! >&2
                    exit 1
                  fi
                else
                  echo IBM Firewall Filter Engine binary not found! >&2
                  exit 1
                fi 
                ;;
  esac
fi

