#!/bin/sh  
# -- BEGIN COPYRIGHT --
# IBM SecureWay Firewall Version 4.1
# Licensed Materials - Property of IBM
#
# 5697-F48
#
# (C) Copyright IBM Corp. 1985, 1994, 1999 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -- END COPYRIGHT --

#
# Component Name: filter_mon.sh
#

PRODUCT=SECUREWAY_FIREWALL

# Description: converts back slashed to forward slashes
#
# Usage: 
#        to_unix $TARGET_PATH TARGET_PATH
to_unix()
{
  if [ "$INTERP" = "w32-ix86" ] ; then
    UNIX_PATH=`echo $1 | sed -e 's|\\\\|/|g'`
  else
    UNIX_PATH=`printf "%s\n" "$1" | sed -e 's|\\\\|/|g'`
  fi
 
  eval $2=$UNIX_PATH
}

REG="reg2.exe"

# ----------------------------------------------------------------------
# Discover if we are on an LCF endpoint.
# ----------------------------------------------------------------------
if [ "x$LCF_BINDIR" != "x" ]; then
  EP=TRUE
  if [ "x$OS" = "xWindows_NT" ]; then
    ETC=$SystemRoot/system32/drivers/etc/Tivoli
    NULL=nul
    if [ -f $TOOLS/$REG ] ; then
      FW_DIR=`$REG -p "SOFTWARE/IBM/IBMFirewall/4.1" | grep RootDirectory | awk -F= '{print $2}'`
      to_unix $FW_DIR FW_DIR
    else
      echo Missing $REG binary.  Please install the plus module
      echo on the Firewall server.
      exit 1
    fi
  else
    ETC=/etc/Tivoli
    NULL=/dev/null
    LANG=en_US
    export LANG
  fi
  TEMP=/tmp
else
  EP=FALSE
fi

if [ x"$EP" = x"FALSE" ] ; then
  if [ -f $SystemRoot/system32/drivers/etc/Tivoli/setup_env.sh ] ; then
          . $SystemRoot/system32/drivers/etc/Tivoli/setup_env.sh
          if [ -f $BINDIR/../generic_unix/TME/PLUS/SECUREWAY_FIREWALL/$REG ] ; then
            FW_DIR=`$BINDIR/../generic_unix/TME/PLUS/SECUREWAY_FIREWALL/$REG -p "SOFTWARE/IBM/IBMFirewall/4.1" | grep RootDirectory | awk -F= '{print $2}'`
            to_unix $FW_DIR FW_DIR
            NLSPATH=$FW_DIR/msg/en_US/%N
            export NLSPATH
          else
            echo Missing $REG binary.  Please install the plus module
            echo on the Firewall server.
            exit 1
          fi
	  TEMP=$DBDIR/tmp
  else
          . /etc/Tivoli/setup_env.sh
	  TEMP=/tmp
          NLSPATH=$NLSPATH:/usr/lib/nls/msg/%L/%N:/usr/lib/nls/msg/%L/%N.cat:/usr/lib/nls/msg/prime/%N
          export NLSPATH
  fi
  OSERV=`objcall 0.0.0 get_oserv`
  INST_DIR=`objcall $OSERV query install_dir`
  PATH=$INST_DIR/generic_unix/TME/PLUS/SECUREWAY_FIREWALL:$BINDIR/AEF:$PATH
  export PATH
fi

  case `uname -sr` in

	"AIX"*)
                if [ -f "/usr/bin/fwfilter" ] ; then 
                  FilterUp=`fwfilter cmd=list | head -n 1 | grep -i active`
                  if [ x"$FilterUp" = x ] ; then
                    echo up
                    exit 0
                  else
                    echo down
                    exit 0
                  fi  
                else
                  echo IBM Firewall Filter Engine binary not found! >&2
                  exit 1
                fi 
		;;

	"Windows_NT 4.0"*)
                if [ -f "$FW_DIR/bin/fwfilter.exe" ] ; then
                  # no head on NT, so grep for full string :-(
                  FilterUp=`$FW_DIR/bin/fwfilter cmd=list | grep "Filter is deactivated"`
                  if [ x"$FilterUp" = x ] ; then
                    echo up
                    exit 0
                  else
                    echo down
                    exit 0
                  fi
                else
                  echo IBM Firewall Filter Engine binary not found! >&2
                  exit 1
                fi
	        ;;
  esac

