#!/usr/local/bin/perl
# -- BEGIN COPYRIGHT --
# IBM SecureWay Firewall Version 4.1
# Licensed Materials - Property of IBM
#
# 5697-F48
#
# (C) Copyright IBM Corp. 1985, 1994, 1999 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -- END COPYRIGHT --

#
# Component Name: Logfile Adapter
#
# $Date: 1999/01/29 17:17:49 $
#
# $Revision: 1.1.1.1 $
#

###########################################
###########################################
#
# Init variables
#
###########################################
###########################################

$_VERSION_RELEASE = "1.0";
$_VERSION_BUILD = "2009";
$_VERSION_BUILDDATE = "Thu Jun  4 14:22:23 CDT 1998";

%errTable = (
    'unktag', 		"Unknown tag/variable name",
    'unkivaltype', 	"Unknown ivalue type",
    'badnumeric',	"Bad numeric format",
    'notnumeric',	"Can not use ! with numeric operator",
    'badfunctionarg',	"Bad function argument",
    'notdefined',	"Usage of a define which is not defined",
    'unkfunctionname',	"Unknown function name",
    'unmatchedquote',	"Unmatched quote",
);

# Class counter 

$defaultClass=0;
$initClass=1;
$firstClass=2;
$nClass=$firstClass;

# Clear/Init variables

%classEvent=();
@RegexVar=();
$classMatchIndex=0;
@prevClassMatchIndex=();
$nsub=0;
%inputVariables=();
$gotINPUT_VARIABLES=0;
$initSC=0;

#  default adapter installation path

# $ADAPTER_HOME='/src/perl/logfileAdapter';

# Trace level

%Level = (
    'FATAL',    0,
    'MAJOR',    1,
    'MINOR',    2,
    'LOW',      3,
    'NORMAL',   4,
    'VERBOSE',  5,
);

$traceid=0;

###########################################
###########################################
#
# MAIN
#
###########################################
###########################################

# --------------------------------------------------------------------------
# Function: getInterpType()
#
# Description:
# --------------------------------------------------------------------------

sub getInterpType
{
  local($rc);

  open(fh, "uname -s 2>nul |");
  $interp = <fh>;
  close(fh);

  chop($interp);

  if    ($interp eq "")           { $interp = "winnt"; }
  elsif ($interp eq "Windows_NT") { $interp = "winnt"; }
  elsif ($interp eq "AIX")        { $interp = "aix";   }
  elsif ($interp eq "SunOS")      { $interp = "sol";   }
  elsif ($interp eq "HP-UX")      { $interp = "hp";    }
  elsif ($interp eq "Linus")      { $interp = "linux"; }
  else  { print STDERR "unknown interp type: $interp\n"; exit 1; }
}

&getInterpType();

# --------------------------------------------------------------------------
# Function: Getopts()
#
# Description:
# --------------------------------------------------------------------------

sub Getopts {
    local($argumentative) = @_;
    local(@args,$_,$first,$rest);
    local($errs) = 0;
    local($[) = 0;

    @args = split( / */, $argumentative );
    while(@ARGV && ($_ = $ARGV[0]) =~ /^-(.)(.*)/) {
	($first,$rest) = ($1,$2);
	$pos = index($argumentative,$first);
	if($pos >= $[) {
	    if($args[$pos+1] eq ':') {
		shift(@ARGV);
		if($rest eq '') {
		    ++$errs unless @ARGV;
		    $rest = shift(@ARGV);
		}
		eval "\$opt_$first = \$rest;";
	    }
	    else {
		eval "\$opt_$first = 1";
		if($rest eq '') {
		    shift(@ARGV);
		}
		else {
		    $ARGV[0] = "-$rest";
		}
	    }
	}
	else {
	    print STDERR "Unknown option: $first\n";
	    ++$errs;
	    if($rest ne '') {
		$ARGV[0] = "-$rest";
	    }
	    else {
		shift(@ARGV);
	    }
	}
    }
    $errs == 0;
}

# --------------------------------------------------------------------------
# Function: getPath()
#
# Description: Take a fully qualified file and return it's path
# --------------------------------------------------------------------------

sub getPath
{
  local($fullPath, *path) = @_;

  $path = "";

  for ($i = length($fullPath); $i > 0; $i--)
  {
    if (substr($fullPath, $i, 1) eq "/" ||
        substr($fullPath, $i, 1) eq "\\")
    {
      $path = substr($fullPath, 0, $i);
      return;
    }
  }

  return;
}


# --------------------------------------------------------------------------
# Function: readConfigFile()
#
# Description: Read the config file and set some variables
# --------------------------------------------------------------------------

sub readConfigFile
{
  local($configFile, *logfile) = @_;
  local($attrib, $value);

  if ($configFile eq "") { return; }

  if (!open(CF, "< $configFile"))
  {
    print "readConfigFile: can't open config file $configFile\n";
    return;
  }

  $_ = <CF>;

  chop;

  ($info, $logfile) = split(/ /, $_);

  $logfile =~ s|\\|/|g;
  close(CF);

  return;
}


# --------------------------------------------------------------------------
# Function: getLogfileFromCONF()
#
# Description: Given a fully qualified path to a .conf file return the
#   logfile it points to.
# --------------------------------------------------------------------------

sub getLogfileFromCONF
{
  local($fullPath, *logfile) = @_;

# &getPath($fullPath, *logfilePath);
  &readConfigFile($fullPath, *logfileName);

# $logfile = $logfilePath . '/' . $logfileName;
  $logfile = $logfileName;

  return;
}

# --------------------------------------------------------------------------
# Function: myDie()
#
# Description: 
# --------------------------------------------------------------------------

sub myDie
{
    local($str)=@_;

    if ($initSC == 1) {
        &Trace($Level{FATAL},$str);
        &termServerConnection(-1);
    }
    else {
        die $str;
    }
}

if (defined($ENV{ADAPTER_HOME})) { $ADAPTER_HOME = $ENV{ADAPTER_HOME}; } 

# init program name

$progname = $0;
while ($progname=~s/^.*\///) {}
while ($progname=~s/^.*\\//) {}
$progname  =~ s/\..*$//;
$PROGNAME=$progname;
$PROGNAME=~ tr/a-z/A-Z/;

# init hostname

$adapter_host = `hostname`;
chop $adapter_host;

# check options

# require $ADAPTER_HOME . '/getopts.pl';
 
&Getopts('a:p:c:m:dhuzTND') || &Usage;
&Usage if defined $opt_h;

# load the communication module 

if ( $opt_a ) {
  $ADAPTER_HOME = $opt_a;
}

if ($ADAPTER_HOME eq "" )
{
  print STDERR "either set environment variable ADAPTER_HOME or\n";
  print STDERR "set is via the -a argument\n";
  exit 1;
}

if ( $opt_z ) {
    print "Adapter ${_VERSION_RELEASE} build ${_VERSION_BUILD} ${_VERSION_BUILDDATE}\n";
    print "(c) Copyright 1998 IT Masters\n";
    exit 0;
}
if ( $opt_N ) {
    # load the fake communication module, for tracing/debugging
    require $ADAPTER_HOME . '/lib/trace.pl';
}
elsif ( $opt_T ) {
    require $ADAPTER_HOME . '/lib/tec.pl';
}
else {			
    require $ADAPTER_HOME . '/lib/tec.pl';
}

# load the specific module
# specificMod is the name of this program minus the heading chars "fw"

$specificMod.=$progname;
$specificMod=~ s/^fw//;
$specificMod= $ADAPTER_HOME . '/lib/'. $specificMod . '.pl';

# print "sp: $specificMod\n";

require $specificMod;

# set our quit signal handler

&setSigHandler();

$TMPdir = "/tmp";

if (! -d $TMPdir)
{
  system("mkdir $TMPdir");
}
# parse the configuration file

&parseConfFile(&getConfigName($progname));

open(OUTFILE, ">$TMPdir/post.cfg");
chmod 0700, 'post.cfg';
print OUTFILE "ServerLocation=$CfgTable{ServerName}\n";
print OUTFILE "ServerPort=$CfgTable{PortNo}\n";
close OUTFILE;
 
if ( $opt_p )
{
  $CfgTable{CONFfile} = $opt_p;
}

if ($CfgTable{CONFfile} eq "")
{
  print STDERR "No .conf file specified in config file or as an CLI arg\n";
  exit 1;
}

&getLogfileFromCONF($CfgTable{CONFfile}, *logfileFromCONF);

if ($logfileFromCONF eq "")
{
  print STDERR "No logfile in .cong file\n";
  exit 1;
}

$CfgTable{LOGFILES} = $logfileFromCONF;

print "lf: $CfgTable{LOGFILES}\n";

# init connection with server

&initServerConnection();

# init Trace stuff

&initTrace();

# init adapter specific 

&InitSpecific();

# parse the class map file

&parseMapFile(&getMapName($progname));

# connect to Server

&connectToServer();

# make me a daemon/service

if ( ! $opt_d ) {
    if ( "$interp" eq "winnt" ) {
        # &makeMeService();
    }
    else {
        require 'POSIX.pm';
        import POSIX.pm;

        &makeMeDaemon();
    }
}

# Main loop

for (;;) {
    if (&ReadSpecific()) {
	if (&matchEvent()) {
	    &sendMessage(&buildMessage(%classEvent));
	}
    } 
    else {
	sleep $CfgTable{PollInterval};
    }
}

###########################################

sub setSigHandler
{
    if ( "$interp" eq "winnt" ) {
        $SIG{'INT'} = 'quitHandler';
        $SIG{'TERM'} = 'quitHandler';
    }
    else {
        $SIG{'INT'} = 'quitHandler';
        $SIG{'TERM'} = 'quitHandler';
        $SIG{'QUIT'} = 'quitHandler';
        $SIG{'KILL'} = 'quitHandler';
        $SIG{'CHLD'} = sub { wait };
    }
}


sub quitHandler {
    local($sig) = @_;

    &Trace($Level{NORMAL},"quitHandler\n");

    FinishSpecific();

    &disconnectFromServer();

    &termServerConnection(0);
    exit 0;
}

#

sub makeMeDaemon
{
    local($pid,$rc);

    if ($pid = fork()){ 
        exit 0; # Parent exits
    }
    if ($pid = fork()){ 
        #while( waitpid( $pid, 0) == -1){
        #    sleep 1;
        #}
        exit 0; # GrandChild exits
    }

    return $! if ($!);

    close STDIN;
    close STDOUT;
    close STDERR;

    chdir "/";

    umask 0;

    # Child continues
    
    $rc = &POSIX::setsid(); # Create a new session

    return "";
}

###########################################
###########################################
#
# std functions
#
###########################################
###########################################


# print usage and exit

sub Usage {
    print STDERR <<EOF;
Usage: $0 [-a path] [-p path] [-c configfile] [-m mapfile] [-d] [-N] [-T] [-D] [-h]
    -a   specify the ADAPTER_HOME env var as a command line option

    -p   specify the path the to .mpf file

    -c   alternate configuration file
	 by default, \$ADAPTER_HOME/conf/$progname.conf is used.

    -m   alternate map file
	 by default, \$ADAPTER_HOME/conf/$progname.map is used.

    -d	 debug (no fork).

    -N   No connection with the server. Should be used for debugging,
	 testing.

    -D   use default class when event not match

    -h   this help
EOF
    exit(1);
}

# get config file name

sub getConfigName {
    local ($programName) = @_;
    local ($file);
    local ($confdir);

    if (defined($ENV{ADAPTER_HOME})) {
        $ADAPTER_HOME = $ENV{ADAPTER_HOME};
    } 

    if ($opt_c) { 		# filled in by Getopts()
	if ( -f $opt_c) {
	    return($opt_c);
	}
        $opt_c = $ADAPTER_HOME . '/conf/' . $opt_c;
        return ($opt_c);
    }

    $confdir = $ADAPTER_HOME . '/conf/' . `uname -n`;
    chop $confdir;

    if ( -d $confdir ) {
        $file = $confdir . '/' . $programName . '.conf';
    } else {
        # the file name is our name + .conf
        $file = $ADAPTER_HOME . '/conf/' . $programName . '.conf';
    }
    return ($file);
}

# get map file name

sub getMapName
{
    local ($programName) = @_;
    local ($file);
    local ($confdir);
    
    if (defined($ENV{ADAPTER_HOME})) {
	$ADAPTER_HOME = $ENV{ADAPTER_HOME};
    } 

    if ($opt_m) { 		# filled in by Getopts()
	if ( -f $opt_m) {
	    return($opt_m);
	}
        $file = $ADAPTER_HOME . '/conf/' . $opt_m;
        return ($file);
    }

    $confdir = $ADAPTER_HOME . '/conf/' . `uname -n`;
    chop $confdir;

    if (-d $confdir ) {
        $file = $confdir . '/' . $progname . '.map';
    } else {
        # the file name is our name + .map
        $file = $ADAPTER_HOME . '/conf/' . $programName . '.map';
    }

    return ($file);
}

# parses the configuration file

sub parseConfFile
{
    local($file) = @_;

    &myDie("Adapter configuration file not found! $file") if ( ! -f $file );

    open(CONFIG, $file);

    # read through the keys
    while (<CONFIG>) {
        chop;
        $_ =~ tr/\r//d;		# drop CR
        next if (/^[ \t]*#/ || /^[ \t]*%/ || /^[ \t]*!/|| /^[ \t]*$/ );
        next unless (/^([a-zA-Z0-9-_]*)[ \t]*=[ \t]*([a-zA-Z0-9-_@\/.:\\; ]*)[ \t]*/);
        $key = $1;
        $val = $2;

        if (defined $CfgTable{$key}) {
	    $tmp=$var;
	    $tmp=~tr/A-Z/a-z/;
	    $tmp=~s/^on$/true/;
	    $tmp=~s/^yes$/true/;
	    if ($tmp eq "true") {
                $CfgTable{$key}="true";
	    } 
	    else {
                $CfgTable{$key}=$val;
	    }
	}
    }
    close(CONFIG);
}

###########################################
###########################################
#
# Parse the map file to create the Class structure
#
###########################################
###########################################

# parse the map (.map) file and construct the Class structure
#
# Format of the Class structure:
#
#Class[#]{name}			= name of the class
#Class[#]{dropevent}		= 1 means drop the event
#Class[#]{nextclass}		= a index to then next NOT subclass 
#				  can also be viewed as a OR when parsing
#Class[#]{subclass}		= a index to the subclass Class[]
#				  can also be viewed as a AND when parsing
#Class[#]{in}[#]{tag}		= name of the tag. Used to parse the specificEvent
#Class[#]{in}[#]{tagindex}	= index from 1-9 . 0 means no index
#Class[#]{in}[#]{not}		= negative operator
#Class[#]{in}[#]{op}		= operator to apply when parsing the specificEvent
#Class[#]{in}[#]{ivalue}	= 
#Class[#]{in}[#]{type}		= 0 means numeric ivalue
#				  1 means string ivalue
#				  2 means regexpr ivalue
#Class[#]{out}[#]{slot}		= name of the slot
#Class[#]{out}[#]{function}	= a function name (printf | equal | ...)
#Class[#]{out}[#]{args}		= 
#
# ? not clear !!! :-( It should help you reading the map.bnf file

sub parseMapFile
{
    local($file) = @_;

    &myDie("Adapter map file not found! $file") if ( ! -f $file );

    open(PARSE, $file);

    &Trace($Level{VERBOSE},"Parsing map file : $file\n");

    # read through the keys
    while (<PARSE>) {
        s/\r//g if (/\r$/);

        next if (/^[ \t]*#/ || /^[ \t]*%/ || /^[ \t]*!/|| /^[ \t]*$/ );

	while ($_=~s/^(.*)[ \t]$/$1/) {}

 	if (/^DEFINE[ \t]+([a-zA-Z0-9-_]+)[ \t]+(.*)$/) {
	    &Trace($Level{VERBOSE},"got define:$1 = $2:\n");
	    $Defined{$1}=$2;
	} 
	elsif (/^[ \t]*INPUT_VARIABLES[ \t]*$/) {
	    &parseINPUT_VARIABLES();
	}
	elsif (/^[ \t]*INIT[ \t]*$/) {
	    &parseINIT();
	}
	elsif (/^[ \t]*DEFAULT[ \t]+([a-zA-Z0-9-_]+)[ \t]*$/) {
	    &parseDEFAULT($1);
	}
	elsif (/^[ \t]*CLASS[ \t]+([a-zA-Z0-9-_]+)[ \t]*(DROP)*[ \t]*$/) {
	    if ($2) {
    	        $Class[$nClass]{dropevent}=1;
	    }
	    else {
    	        $Class[$nClass]{dropevent}=0;
	    }

	    $c=&parseCLASS($nClass,$1);
	    $nClass++;
    	    $Class[$c]{nextclass}=$nClass;
	}
	else {
	    select(PARSE); 
	    &myDie("Error parsing the map file at line:$. Unknown statement\n-->>$_");
	}
    }

    &Trace($Level{VERBOSE},"done\n");

    if ( (! $Class[$defaultClass]) && $opt_D ) {
	&myDie("Error: no DEFAULT class found in the map file.\n");
    }

    # debug
    if ($CfgTable{Trace} eq "true") {
	&Trace($Level{VERBOSE},"Class Structure:\n");
        &printClassStructure($firstClass,' ');
    }

    close(PARSE);
}

# parse the INIT statement

sub parseINIT
{
    &myDie("Error parsing the map file: no INPUT_VARIABLES definition!\n") if ($gotINPUT_VARIABLES == 0);

    &Trace($Level{VERBOSE},"got init\n");

    &parseMAP($initClass);
}

# parse the INPUT_VARIABLES

sub parseINPUT_VARIABLES
{
    local($n);

    &Trace($Level{VERBOSE},"got INPUT_VARIABLES:\n");

    while (<PARSE>) {
        s/\r//g if (/\r$/);
        next if (/^[ \t]*#/ || /^[ \t]*%/ || /^[ \t]*!/|| /^[ \t]*$/ );

	while ($_=~s/^(.*)[ \t]$/$1/) {}

 	if (/^[ \t]*\$([a-zA-Z0-9-_]+)[ \t]*(0?-?([0-9i]*))[ \t]*$/) {
	    if ($2) {
	        for ($n=0; $n<=($3 eq "i"?99:$3); $n++) {
        	    $inputVariables{$1 . $n}=1;
	        }
            }
            else {
	        $inputVariables{$1}=1;
            }
	}
 	elsif (/^[ \t]*END[ \t]*$/) {

    	    $gotINPUT_VARIABLES=1;

	    return;
	}
	else {
	    select(PARSE); 
	    &myDie("Error parsing the map file at line:$. Unknown statement\n-->>$_");
	}
    }
}

# parse the DEFAULT statement

sub parseDEFAULT
{
    local($className)=@_;

    &myDie("Error parsing the map file: no INPUT_VARIABLES definition!\n") if ($gotINPUT_VARIABLES == 0);

    &Trace($Level{VERBOSE},"got default:$className:\n");

    $Class[$defaultClass]{name}=$className;

    &parseMAP($defaultClass);

    if ( ! $Class[$defaultClass]{out}[0]) {
	&myDie("Error parsing the map file: no valid lines found in the DEFAULT statement.\n");
    }
}

# parse the CLASS statement

sub parseCLASS
{
    local($c,$className)=@_;
    local($i)=0;
    local($o)=0;
    local($s)=0;
    local($sub);

    &myDie("Error parsing the map file: no INPUT_VARIABLES definition!\n") if ($gotINPUT_VARIABLES == 0);

    &Trace($Level{VERBOSE},"got class:$className:\n");

    $Class[$c]{name}=$className;

    while (<PARSE>) {
        s/\r//g if (/\r$/);
        next if (/^[ \t]*#/ || /^[ \t]*%/ || /^[ \t]*!/|| /^[ \t]*$/ );

	while ($_=~s/^(.*)[ \t]$/$1/) {}

 	if (/^[ \t]*\$([a-zA-Z0-9-_]+[\[1-9\]]*)[ \t]+([a-z_]+)[ \t]+(.*)$/) {
	    &parseCondition($c,$i,$1,$2,$3);
	    $i++;
	}
	elsif (/^[ \t]*CLASS[ \t]+([a-zA-Z0-9-_]+)[ \t]*(DROP)*[ \t]*$/) {
	    $nClass++;

	    if ($2) {				# if CLASS to DROP 
    	        $Class[$nClass]{dropevent}=1;
	    }
	    else {
    	        $Class[$nClass]{dropevent}=0;
	    }

	    if ($s > 0) {
    	        $Class[$Class[$c]{subclass}[$s-1]]{nextclass}=$nClass;
	    }
		
	    $sub=&parseCLASS($nClass,$1);
    	    $Class[$c]{subclass}[$s]=$sub;
	    $s++;
	}
 	elsif (/^[ \t]*([a-zA-Z0-9-_]+)[ \t]*=[ \t]*([a-z]+)[ \t]*\([ \t]*(.*)[ \t]*\)$/) {
	    &parseMAPline($c,$o,$1,$2,$3);
	    $o++;
	}
 	elsif (/^[ \t]*([a-zA-Z0-9-_]+)[ \t]*=[ \t]*(.*)$/) {
	    &parseMAPline($c,$o,$1,'equal',$2);
	    $o++;
	}
 	elsif (/^[ \t]*END[ \t]*$/) {
	    return($c);
	}
	else {
	    select(PARSE); 
	    &myDie("Error parsing the map file at line:$. Unknown statement\n-->>$_");
	}
    }
}

# parse the Condition line

sub parseCondition
{
    local($c,$i,$tag,$op,$ivalue)=@_;
    local($res,$errindex);

    ($res,$errindex)=&validCondition($tag,$op,$ivalue) ;
    if ($res == 1) {
	select(PARSE); 
	&myDie("Error parsing the map file at line:$. $errTable{$errindex}\n-->>$_");
    }

    # check if ivalue is a DEFINE
    if ($ivalue =~ s/^#(.*)$/$1/) {
        $ivalue=$Defined{$ivalue};
    }

    # check if the tag is tag[0-9] or just the tag

    if ($tag =~ /^([a-zA-Z0-9-_]+)\[([1-9])\]$/) {
        $Class[$c]{in}[$i]{tag}=$1;
        $Class[$c]{in}[$i]{tagindex}=$2;
    }
    else {
        $Class[$c]{in}[$i]{tag}=$tag;
        $Class[$c]{in}[$i]{tagindex}=0;
    }

    # save and convert the operator
    # convert it to a smaller string: faster for string match

    $Class[$c]{in}[$i]{not}='';

    if ($op eq 'equals') {
        $Class[$c]{in}[$i]{op}='=';
    }
    elsif ($op eq 'not_equals') {
        $Class[$c]{in}[$i]{op}='=';
        $Class[$c]{in}[$i]{not}='!';
    }
    elsif ($op eq 'small') {
        $Class[$c]{in}[$i]{op}='<';
    }
    elsif ($op eq 'greater') {
        $Class[$c]{in}[$i]{op}='>';
    }
    elsif ($op eq 'smaller_or_equals') {
        $Class[$c]{in}[$i]{op}='<=';
    }
    elsif ($op eq 'greater_or_equals') {
        $Class[$c]{in}[$i]{op}='>=';
    }

    #

    $Class[$c]{in}[$i]{ivalue}=$ivalue;

    # check ivalue type
    # Class[#]{in}[#]{type}	= 0 means numeric ivalue
    #				  1 means string ivalue
    #				  2 means regexpr ivalue
    if (&isNumeric($ivalue)) {
        $Class[$c]{in}[$i]{type}=0;
    }
    elsif (&isString($ivalue)) {
        $Class[$c]{in}[$i]{type}=1;
        $Class[$c]{in}[$i]{ivalue}=~s/^"(.*)"$/$1/; # remove head&trail quotes
    }
    elsif (&isRegexpr($ivalue)) {
        $Class[$c]{in}[$i]{type}=2;
        $Class[$c]{in}[$i]{ivalue}=~s/^\/(.*)\/$/$1/; # remove head&trail slash
    }

    &Trace($Level{VERBOSE},"got :$tag:$op:$Class[$c]{in}[$i]{ivalue}:\n");
}

# validate the Condition line according $op and $ivalue

sub validCondition
{
    local($tag,$op,$ivalue)=@_;

    if ( ! &validTag($tag)) {
	return(1,'unktag');
    }

    if ($op eq 'equals' || $op eq 'not_equals') {
	if ( !( &isNumeric($ivalue)||&isString($ivalue)||&isRegexpr($ivalue)||&isDefine($ivalue))) {
	    return(1,'unkivaltype');
	}
	elsif (&isDefine($ivalue)) {
            $ivalue =~ s/^#(.*)$/$1/;
	    if (! exists $Defined{$ivalue}) {
                return(1,'notdefined');
	    }
	}
    }
    elsif ($op eq 'smaller' || $op eq 'greater' || $op eq 'smaller_or_equals' || $op eq 'greater_or_equals') {
        if ( ! &isNumeric($ivalue)) {
	    return (1,'badnumeric');
	}
    }
    return(0,'');
}

# parse the mapping statement 
# called for the DEFAULT or the INIT statement

sub parseMAP
{
    local($c)=@_;
    local($o)=0;

    while (<PARSE>) {
        s/\r//g if (/\r$/);
        next if (/^[ \t]*#/ || /^[ \t]*%/ || /^[ \t]*!/|| /^[ \t]*$/ );

	while ($_=~s/^(.*)[ \t]$/$1/) {}

 	if (/^[ \t]*([a-zA-Z0-9-_]+)[ \t]*=[ \t]*([a-z]+)[ \t]*\([ \t]*(.*)[ \t]*\)$/) {
	    &parseMAPline($c,$o,$1,$2,$3);
	    $o++;
	}
 	elsif (/^[ \t]*([a-zA-Z0-9-_]+)[ \t]*=[ \t]*(.*)$/) {
	    &parseMAPline($c,$o,$1,'equal',$2);
	    $o++;
	}
 	elsif (/^[ \t]*END[ \t]*$/) {
	    return(0);
	}
	else {
	    select(PARSE); 
	    &myDie("Error parsing the map file at line:$. Unknown statement\n-->>$_");
	}
    }
}

# parse the mapping line

sub parseMAPline
{
    local($c,$o,$slot,$function,$args)=@_;
    local($res,$errindex,$i,$err);
    local(@pargs);

    # remove trailing blanks
    $args=~s/^(.*)[ \t]+$/$1/;

    ($res,$errindex)=&validMAPline($function,$args);
    if ($res == 1) {
	select(PARSE); 
	&myDie("Error parsing the map file at line:$. $errTable{$errindex}\n-->>$_");
    }

    $Class[$c]{out}[$o]{slot}=$slot;
    $Class[$c]{out}[$o]{function}=$function;

    if ($function =~ /equal/) {
        if ($args =~ s/^#(.*)$/$1/) {
            $args=$Defined{$args};	# replace the DEFINE by its value
        }
        $Class[$c]{out}[$o]{args}=$args;
    }
    elsif ($function =~ /upper/ || $function =~ /lower/) {
        $Class[$c]{out}[$o]{args}=$args;
    }
    elsif ($function =~ /gethostbyname/) {
        $Class[$c]{out}[$o]{args}=$args;
    }
    elsif ($function =~ /gethostbyaddr/) {
        $Class[$c]{out}[$o]{args}=$args;
    }
    elsif ($function =~ /substr/) {
        $args =~ /^(\$[a-zA-Z0-9-_]+[\[1-9\]]*)[ \t]*,[ \t]*([0-9]+)[ \t]*,?[ \t]*([0-9]*)$/;
    
        $Class[$c]{out}[$o]{args}=$1;	# tag name
        $Class[$c]{out}[$o]{arg1}=$2;	# start position
        $Class[$c]{out}[$o]{arg2}=$3;	# len
    }
    elsif ($function =~ /printf/) {
	($err,@pargs) = &myquotewords("[ \t]*,[ \t]*",0,$args);

	for ($i=0;$pargs[$i];$i++) {
       	    if ($pargs[$i] =~ s/^#([a-zA-Z0-9-_]+)/$1/) {
		# if there are DEFINE replace them with their value
		$pargs[$i]=$Defined{$pargs[$i]};
    	    } 
	}

	$args=join(',', @pargs);
        $Class[$c]{out}[$o]{args}=$args;
    }

    &Trace($Level{VERBOSE},"got :$slot:$function:$args:\n") if ($opt_N);
}

# validate the MAP line according $op and $ivalue

sub validMAPline
{
    local($function,$args)=@_;
    local(@pargs);
    local($i,$err);

    if ($function =~ /equal/) {
        if (!(&isTag($args)||isString($args)||&isNumeric($args)||&isDefine($args))) {
            return(1,'badfunctionarg');
	}
	elsif (&isDefine($args)) {
            $args =~ s/^#(.*)$/$1/;
	    if (! exists $Defined{$args}) {
                return(1,'notdefined');
	    }
	}
	elsif (&isTag($args)) {
	    if (! &validTag($args)) {
                return(1,'unktag');
	    }
	}
    }
    elsif ($function =~ /lower/ || $function =~ /upper/) {
        if (!(&isTag($args) || &isString($args))) {
            return(1,'badfunctionarg');
	}
	elsif (&isTag($args)) {
	    if (! &validTag($args)) {
                return(1,'unktag');
	    }
        }
    }
    elsif ($function =~ /gethostbyname/) {
        if (! &isTag($args)) {
            return(1,'badfunctionarg');
	}
    }
    elsif ($function =~ /gethostbyaddr/) {
        if (! &isTag($args)) {
            return(1,'badfunctionarg');
	}
    }
    elsif ($function =~ /substr/) {
        if (!($args =~ /^(\$[a-zA-Z0-9-_]+[\[1-9\]]*)[ \t]*,[ \t]*[0-9]+[ \t]*,?[ \t]*[0-9]*$/)) {
            return(1,'badfunctionarg');
	}
	elsif (! &validTag($1)) {
                return(1,'unktag');
        }
    }
    elsif ($function =~ /printf/) {

	($err,@pargs) = &myquotewords("[ \t]*,[ \t]*",0,$args);
	return(1,'unmatchedquote') if $err;

	for ($i=0;$pargs[$i];$i++) {
    	    if ($pargs[$i] =~ s/^#([a-zA-Z0-9-_]+)/$1/)
    	    {
		if ( ! $Defined{$pargs[$i]}) {
            	    return(1,'notdefined');
		}
    	    }
	}
    }
    else {
        return(1,'unkfunctionname');
    }
    return(0,'');
}

######################################################
#
# this function has been stooled from the std Perl distribution,
# and a bit modified to fit better with this program
#

sub myquotewords {

# The inner "for" loop builds up each word (or $field) one $snippet
# at a time.  A $snippet is a quoted string, a backslashed character,
# or an unquoted string.  We fall out of the "for" loop when we reach
# the end of $_ or when we hit a delimiter.  Falling out of the "for"
# loop, we push the $field we've been building up onto the list of
# @words we'll be returning, and then loop back and pull another word
# off of $_.
#
# The first two cases inside the "for" loop deal with quoted strings.
# The first case matches a double quoted string, removes it from $_,
# and assigns the double quoted string to $snippet in the body of the
# conditional.  The second case handles single quoted strings.  In
# the third case we've found a quote at the current beginning of $_,
# but it didn't match the quoted string regexps in the first two cases,
# so it must be an unbalanced quote and we croak with an error (which can
# be caught by eval()).
#
# The next case handles backslashed characters, and the next case is the
# exit case on reaching the end of the string or finding a delimiter.
#
# Otherwise, we've found an unquoted thing and we pull of characters one
# at a time until we reach something that could start another $snippet--
# a quote of some sort, a backslash, or the delimiter.  This one character
# at a time behavior was necessary if the delimiter was going to be a
# regexp (love to hear it if you can figure out a better way).

    local($delim, $keep, @lines) = @_;
    local(@words,$snippet,$field,$_);

    $_ = join('', @lines);
    while (length($_)) {
	$field = '';
	for (;;) {
            $snippet = '';
	    # modified to keep quote     if (s/^"(([^"\\]|\\[\\"])*)"//) {
	    if (s/^("([^"\\]|\\[\\"])*")//) {
		$snippet = $1;
                $snippet = "\"$snippet\"" if ($keep);
	    }
	    # modified for the same reason elsif (s/^'(([^'\\]|\\[\\'])*)'//) {
	    elsif (s/^('([^'\\]|\\[\\'])*')//) {
		$snippet = $1;
                $snippet = "'$snippet'" if ($keep);
	    }
	    elsif (/^["']/) {
		# modified to return an error code
		return(1,'');
	    }
            elsif (s/^\\(.)//) {
                $snippet = $1;
                $snippet = "\\$snippet" if ($keep);
            }
	    elsif (!length($_) || s/^$delim//) {
               last;
	    }
	    else {
                while ($_ && !(/^$delim/ || /^['"\\]/)) {
		   $snippet .=  substr($_, 0, 1);
                   substr($_, 0, 1) = '';
                }
	    }
	    $field .= $snippet;
	}
	push(@words, $field);
    }
    # modified to return an error code
    return(0,@words);
}

######################################################

sub isTag
{
    local($arg)=@_;

    return(1) if ($arg =~ /^\$[a-zA-Z0-9-_]*[\[1-9\]]*$/) ;
    return(0);
}

sub isDefine
{
    local($arg)=@_;

    return(1) if ($arg =~ /^#.*$/) ;
    return(0);
}

sub isNumeric
{
    local($arg)=@_;

    return(1) if ($arg =~ /^[0-9-+\.,]*$/) ;
    return(0);
}

sub isRegexpr
{
    local($arg)=@_;

    return(1) if ($arg =~ /^\/.*\/$/) ;
    return(0);
}

sub isString
{
    local($arg)=@_;

    return(1) if ($arg =~ /^".*"$/) ;
    return(0);
}

#

sub validTag
{
    local($tag)=@_;

    $tag =~ /^\$?([a-zA-Z0-9-_]+)/;
    if (defined $inputVariables{$1}) {
	return(1);
    }
    else {
        return(0);
    }
}

# print class structure

sub printClassStructure
{
    local($c,$sep)=@_;
    local($i)=0;

    return if ( ! $Class[$c] );

    &Trace($Level{VERBOSE},"$sep $Class[$c]{name}\n");

    if ($Class[$c]{subclass}[0]) {
        &printClassStructure($Class[$c]{subclass}[0],$sep. '  ');
    }
    if ($Class[$c]{nextclass}) {
        &printClassStructure($Class[$c]{nextclass},$sep);
    }
}

###########################################
###########################################
#
# Create the MC event. Match event against the Class structure
#
###########################################
###########################################

# match a specificEvent event with a class

sub matchEvent
{
    #debug
    &printSpecificEvent() if ($CfgTable{Trace} eq "true") ;

    %classEvent=();			# clear
    @RegexVar=();
    $classMatchIndex=0;
    @prevClassMatchIndex=();
    $nsub=0;

    &initEvent();

    if (&matchAndCreateEvent($firstClass)) {

	#debug
        if ($CfgTable{Trace} eq "true") {
	    if ( $Class[$classMatchIndex]{dropevent}) {
                &Trace($Level{VERBOSE},"-->event dropped.\n\n");
            }
            else {
                local(@slots);

                &Trace($Level{VERBOSE},"-->event matched:$classEvent{classname}.\n");
	        if ($opt_N) {
                    foreach $slot (keys %classEvent) {
                        &Trace($Level{VERBOSE},"    $slot=$classEvent{$slot}\n");
                    }
		}
            }
        }

	return( ! $Class[$classMatchIndex]{dropevent} );
    }
    elsif ( $opt_D ) {
	&useDefaultClass();

        &Trace($Level{VERBOSE},"-->event matched with DEFAULT class.\n");

        return(1);
    }
    else {

        &Trace($Level{VERBOSE},"-->event not matched.\n");

        return(0);
    }
}

# init event

sub initEvent
{
    &mapOUT($initClass);
}

# match a specificEvent event with a class and create the MC event 

sub matchAndCreateEvent
{
    local($c) = @_;
    local($i,$o,$r,$rr);

    # Evaluate the Condition

    for ($i=0,$rr=1;$Class[$c]{in}[$i] && $rr==1;$i++) {

        if (defined $specificEvent{$Class[$c]{in}[$i]{tag}}) {
	    $r=&evalCondition($c,$i);
	} 
	else {
	    $r=0;
	}
	$rr=$r;   
    }

    # $r == 1 if ALL Condition are satisfied

    if ($r == 1) {

        $classEvent{classname}=$Class[$c]{name};
	
        # save the index that matched the event

	$classMatchIndex=$c;
        $prevClassMatchIndex[$nsub]=$c;
	$nsub++;

	&mapOUT($c);

	# is there any sub class ?

        if ($Class[$c]{subclass}[0]) {
	    &matchAndCreateEvent($Class[$c]{subclass}[0]);
        }
	return(1);	# MATCHED 
    }

    # current class is not the good one, try with the next one if there is

    if ($Class[$c]{nextclass}) {
	return(&matchAndCreateEvent($Class[$c]{nextclass}));
    }
    return(0);		# not MATCHED
}

# evaluate the Condition expression
# return 1 when match
# return 0 when not match

sub evalCondition
{
    local($c,$i)=@_;

    # check if the tag is a tag name or if it is a tag name with index.
    # which is completely different. For the first, we just use the value
    # from the $specificEvent{} but for the second one, we have to use
    # a variable from a previous regular expression.

    if ($Class[$c]{in}[$i]{tagindex} == 0) {
        return(&evalWithSpecificEvent($c,$i));
    }
    else {
        return(&evalWithRegexVar($c,$i));
    }
}

# evaluate the Condition expression
# return 1 when match
# return 0 when not match

sub evalWithSpecificEvent
{
    local($c,$i)=@_;
    local($j);

    #Class[#]{in}[#]{tag}		= name of the tag. Used to parse the specificEvent
    #Class[#]{in}[#]{tagindex}		= index from 1-9 . 0 means no index
    #Class[#]{in}[#]{not}		= negative operator
    #Class[#]{in}[#]{op}		= operator to apply when parsing the specificEvent
    #Class[#]{in}[#]{ivalue}
    #Class[#]{in}[#]{type}		= 0 means numeric ivalue
    #				  	  1 means string ivalue
    #				  	  2 means regexpr ivalue
    #RegexVar[$c]{aTagName}[1-9]	= the regular expression variable
    
    if ($Class[$c]{in}[$i]{op} eq '=') {
	
	# numeric ivalue ?

        if ($Class[$c]{in}[$i]{type} == 0) {
            if ($specificEvent{$Class[$c]{in}[$i]{tag}} == $Class[$c]{in}[$i]{ivalue}) {
		return(($Class[$c]{in}[$i]{not}?0:1));
	    }
	    return(($Class[$c]{in}[$i]{not}?1:0));
	}
	
	# string ivalue ?

        elsif ($Class[$c]{in}[$i]{type} == 1) {
            if ($specificEvent{$Class[$c]{in}[$i]{tag}} eq $Class[$c]{in}[$i]{ivalue}) {
		return(($Class[$c]{in}[$i]{not}?0:1));
	    }
	    return(($Class[$c]{in}[$i]{not}?1:0));
	}
	
	# regexpr ivalue ?

        elsif ($Class[$c]{in}[$i]{type} == 2) {
            if ($specificEvent{$Class[$c]{in}[$i]{tag}} =~ /$Class[$c]{in}[$i]{ivalue}/) {

		# save the subpattern from the corresponding set of parentheses
		# in the previous patern match.
	        for ($j=1;$j<10;$j++) {
		    $RegexVar[$c]{$Class[$c]{in}[$i]{tag}}[$j]=$$j;
		}

		return(($Class[$c]{in}[$i]{not}?0:1));
	    }
	    return(($Class[$c]{in}[$i]{not}?1:0));
	}
    }
    elsif ($Class[$c]{in}[$i]{op} eq '<') {
        if ($specificEvent{$Class[$c]{in}[$i]{tag}} < $Class[$c]{in}[$i]{ivalue}) {
	    return(1);
        }
    }
    elsif ($Class[$c]{in}[$i]{op} eq '>') {
        if ($specificEvent{$Class[$c]{in}[$i]{tag}} > $Class[$c]{in}[$i]{ivalue}) {
	    return(1);
        }
    }
    elsif ($Class[$c]{in}[$i]{op} eq '<=') {
        if ($specificEvent{$Class[$c]{in}[$i]{tag}} <= $Class[$c]{in}[$i]{ivalue}) {
	    return(1);
        }
    }
    elsif ($Class[$c]{in}[$i]{op} eq '>=') {
        if ($specificEvent{$Class[$c]{in}[$i]{tag}} >= $Class[$c]{in}[$i]{ivalue}) {
	    return(1);
        }
    }
    return(0);
}

# evaluate the Condition expression
# return 1 when match
# return 0 when not match

sub evalWithRegexVar
{
    local($c,$i)=@_;
    local($j);

    #Class[#]{in}[#]{tag}		= name of the tag. Used to parse the specificEvent
    #Class[#]{in}[#]{tagindex}		= index from 1-9 . 0 means no index
    #Class[#]{in}[#]{not}		= negative operator
    #Class[#]{in}[#]{op}		= operator to apply when parsing the specificEvent
    #Class[#]{in}[#]{ivalue}
    #Class[#]{in}[#]{type}		= 0 means numeric ivalue
    #				  	  1 means string ivalue
    #				  	  2 means regexpr ivalue
    #RegexVar[$c]{aTagName}[1-9]	= the regular expression variable
    
    if ($Class[$c]{in}[$i]{op} eq '=') {
	
	# numeric ivalue ?

        if ($Class[$c]{in}[$i]{type} == 0) {
            if (&getRegexVarValue($c,$Class[$c]{in}[$i]{tag},$Class[$c]{in}[$i]{tagindex}) == $Class[$c]{in}[$i]{ivalue}) {
		return(($Class[$c]{in}[$i]{not}?0:1));
	    }
	    return(($Class[$c]{in}[$i]{not}?1:0));
	}
	
	# string ivalue ?

        elsif ($Class[$c]{in}[$i]{type} == 1) {
            if (&getRegexVarValue($c,$Class[$c]{in}[$i]{tag},$Class[$c]{in}[$i]{tagindex}) eq $Class[$c]{in}[$i]{ivalue}) {
		return(($Class[$c]{in}[$i]{not}?0:1));
	    }
	    return(($Class[$c]{in}[$i]{not}?1:0));
	}
	
	# regexpr ivalue ?

        elsif ($Class[$c]{in}[$i]{type} == 2) {
            if (&getRegexVarValue($c,$Class[$c]{in}[$i]{tag},$Class[$c]{in}[$i]{tagindex}) =~ /$Class[$c]{in}[$i]{ivalue}/) {

		# save the subpattern from the corresponding set of parentheses
		# in the previous patern match.
	        for ($j=1;$j<10;$j++) {
		    $RegexVar[$c]{$Class[$c]{in}[$i]{tag}}[$j]=$$j;
		}

		return(($Class[$c]{in}[$i]{not}?0:1));
	    }
	    return(($Class[$c]{in}[$i]{not}?1:0));
	}
    }
    elsif ($Class[$c]{in}[$i]{op} eq '<') {
        if (&getRegexVarValue($c,$Class[$c]{in}[$i]{tag},$Class[$c]{in}[$i]{tagindex}) < /$Class[$c]{in}[$i]{ivalue}/) {
	    return(1);
        }
    }
    elsif ($Class[$c]{in}[$i]{op} eq '>') {
        if (&getRegexVarValue($c,$Class[$c]{in}[$i]{tag},$Class[$c]{in}[$i]{tagindex}) > /$Class[$c]{in}[$i]{ivalue}/) {
	    return(1);
        }
    }
    elsif ($Class[$c]{in}[$i]{op} eq '<=') {
        if (&getRegexVarValue($c,$Class[$c]{in}[$i]{tag},$Class[$c]{in}[$i]{tagindex}) <= /$Class[$c]{in}[$i]{ivalue}/) {
	    return(1);
        }
    }
    elsif ($Class[$c]{in}[$i]{op} eq '>=') {
        if (&getRegexVarValue($c,$Class[$c]{in}[$i]{tag},$Class[$c]{in}[$i]{tagindex}) >= /$Class[$c]{in}[$i]{ivalue}/) {
	    return(1);
        }
    }
    return(0);
}

# retreive the previous valid RegexVar variable

sub getRegexVarValue
{
    local($c,$tag,$tagindex)=@_;
    local($n);

    if (defined $RegexVar[$c]{$tag}[$tagindex]) {
	return($RegexVar[$c]{$tag}[$tagindex]);
    }

    for ($n=$nsub-1; $n >= 0; $n--) {
        if (defined $RegexVar[$prevClassMatchIndex[$n]]{$tag}[$tagindex]) {
	    return($RegexVar[$prevClassMatchIndex[$n]]{$tag}[$tagindex]);
        }
    }
    
    # nothing found. Because of an error in the class definition file!

    return("");
}

# do the map out
# fill in the slot=value 

sub mapOUT
{
    local($c)=@_;
    local($o);

    #Class[#]{out}[#]{slot}	= name of the slot
    #Class[#]{out}[#]{function}	= a function name (printf | equal | ...)

    for ($o=0;$Class[$c]{out}[$o];$o++) {
	
	# call the function evalxxx()

	$classEvent{$Class[$c]{out}[$o]{slot}}= &{'eval' . $Class[$c]{out}[$o]{function}} ($c,$o);

    }

}

sub evalequal
{
    local($c,$o)=@_;

    if (&isTag($Class[$c]{out}[$o]{args})) {
        return(&getTagVal($Class[$c]{out}[$o]{args},$c));
    }
    elsif (&isNumeric($Class[$c]{out}[$o]{args})) {
        return($Class[$c]{out}[$o]{args});
    }
    elsif (&isString($Class[$c]{out}[$o]{args})) {
        return($Class[$c]{out}[$o]{args});
    }
}

sub evallower
{
    local($c,$o)=@_;
    local($res);

    if (&isTag($Class[$c]{out}[$o]{args})) {
        $res= &getTagVal($Class[$c]{out}[$o]{args},$c);
    }
    elsif (&isString($Class[$c]{out}[$o]{args})) {
        $res=$Class[$c]{out}[$o]{args};
    }
    $res=~ tr/A-Z/a-z/;
    return($res);
}

sub evalupper
{
    local($c,$o)=@_;
    local($res);

    if (&isTag($Class[$c]{out}[$o]{args})) {
        $res= &getTagVal($Class[$c]{out}[$o]{args},$c);
    }
    elsif (&isString($Class[$c]{out}[$o]{args})) {
        $res=$Class[$c]{out}[$o]{args};
    }
    $res=~ tr/a-z/A-Z/;
    return($res);
}

sub evalgethostbyname
{
    local($c,$o)=@_;
    local($name,$aliases,$type,$len,$addr,$res);

    ($name,$aliases,$type,$len,$addr)=gethostbyname(&getTagVal($Class[$c]{out}[$o]{args},$c));

    $res=inet_ntoa($addr);

    return($res);
}

sub evalgethostbyaddr
{
    local($c,$o)=@_;
    local($addr,$res);

    $addr=inet_aton(&getTagVal($Class[$c]{out}[$o]{args},$c));

    ($res,,,,)=gethostbyaddr($addr,AF_INET);

    return($res);
}

sub evalsubstr
{
    local($c,$o)=@_;

    if ($Class[$c]{out}[$o]{arg2}) {		# do we got a lenght
	return(substr(&getTagVal($Class[$c]{out}[$o]{args},$c),$Class[$c]{out}[$o]{arg1},$Class[$c]{out}[$o]{arg2}));
    }
    else {
	return(substr(&getTagVal($Class[$c]{out}[$o]{args},$c),$Class[$c]{out}[$o]{arg1}));
    }
}

sub evalprintf
{
    local($c,$o)=@_;
    local($i,$printformat,@pargs,$err);

    ($err,@pargs) = &myquotewords("[ \t]*,[ \t]*",0,$Class[$c]{out}[$o]{args});

    for ($i=0;$pargs[$i];$i++) {
        if ($pargs[$i] =~ s/^(\$[a-zA-Z0-9-_]+[\[1-9\]]*)/$1/) {

	    # manage $tag argument
	    $pargs[$i]=&getTagVal($pargs[$i],$c);
         }
    }

    $printformat=$pargs[0];
    # remove the printf format argument from the @pargs list
    shift @pargs;

    return(sprintf $printformat,@pargs);
}

# return the tag value or the tag variable value

sub getTagVal
{
    local($tag,$c)=@_;
    local($tagname);

    $tag =~ /^\$([a-zA-Z0-9-_]*).*$/;
    $tagname=$1;
    if ($tag =~ /^\$[a-zA-Z0-9-_]*\[([1-9])\]$/) {
        return(&getRegexVarValue($c,$tagname,$1));
    }
    else {
	return($specificEvent{$tagname});
    }
}


sub printSpecificEvent
{
    local($k,$v);

 return;

    &Trace($Level{VERBOSE},"Received this specific event:\n");

    while (($k,$v)=each %specificEvent) {
	&Trace($Level{VERBOSE},"\t$k=$v\n");
    }
}


# use the default class to create the event

sub useDefaultClass
{
    $classEvent{classname}=$Class[$defaultClass]{name};

    &mapOUT($defaultClass);
}
