#!/etc/Tivoli/bin/perl
# -- BEGIN COPYRIGHT --
# IBM SecureWay Firewall Version 4.1
# Licensed Materials - Property of IBM
#
# 5697-F48
#
# (C) Copyright IBM Corp. 1985, 1994, 1999 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -- END COPYRIGHT --

#
# Component Name: Logfile Adapter Configuration File Setup
#
# Program: install.pl
#

require 'getopts.pl';
require 'readConfigFile.pl';
require 'updateAixSysFiles.pl';
#require 'updateHpSysFiles.pl';
#require 'updateSolSysFiles.pl';

$PROD        = "itcell";	# what the server executable will be called
$RB_DIR_NAME = "rb";		# the name of the class/rule base directory
$RULES_DIR_NAME = "RULES";    # the name of the rules directory
$CLASS_DIR_NAME = "TEC_CLASSES";  # the name of the class directory

$SERV_MGR     = "servMgr";	# NT services manager
$ENV_DIR_PATH = "/etc/Tivoli/lfa"; # where to put well known setup file

# --------------------------------------------------------------------------
# Function: usage()
#
# Description: print the usage message and exit
#
# -s srcBinDir
# -g srcCfgDir
# -b binDir
# -c configDir
# -r PERL_LOC
# -o owner       owner of files in target directory (default: root)
# -f configFile  install config file to uses (default: install.cfg)
# -x y/n         over right files/directories (default: y)
# -u y/n         update system files for auto-start at boot (default: y)
# -d             debug option
# -h             this help message
# 
# --------------------------------------------------------------------------

sub usage
{
  print "$0: [-s srcBinDir] [-g srcCfgDir] [-b binDir] [-c configDir]\n";
  printf"		[-f optionsFile] [-o owner] [-x y/n] [-u y/n] [-h]\n\n";
  print "where:\n";
  print "  -s srcBinDir source binaries dir\n";
  print "  -g srcCfgDir source config dir (default: =srcBinDir)\n";
  print "  -b binDir    target dir for executables (default: /etc/Tivoli/lfa)\n";
  print "  -c configDir target dir for config file (default: =binDir)\n";
  print "  -f filename  install config file (default: install.cfg)\n";
  print "  -o owner     owner of files in target directory (default: root)\n";
  print "  -x y/n       over right files/directories (default: y)\n"; 
  print "  -u y/n       update system files for auto-start at boot (def: y)\n"; 
  print "  -d           debug option\n"; 
  print "  -h           this help message\n"; 
  print "\n";
  print "$0 -s /cdrom\n";
  print "$0 -f install.cfg\n";

  exit;
}

# --------------------------------------------------------------------------
# Function: bailOnError()
#
# Description: 
# --------------------------------------------------------------------------

sub bailOnError
{
  local($rc, $msg) = @_;

  if ($rc != 0)
  {
    print STDERR "$msg";
    exit(1);
  }
}

# --------------------------------------------------------------------------
# Function: trace()
#
# Description: 
# --------------------------------------------------------------------------

sub trace
{
  local($msg) = @_;

  if ($TRACEON != 0) { print "$msg"; }
}

# --------------------------------------------------------------------------
# Function: parseCmdLine()
#
# Description: Parse the command line. This will over-write the config file.
# --------------------------------------------------------------------------

sub parseCmdLine
{
  local(*srcDir, *sCfgDir, *binDir, *configDir, *PERL_LOC, *optionFile,
	*owner, *overRight, *updateSys) = @_;

  &Getopts('s:b:g:c:r:f:o:x:u:dh');

  if (defined $opt_s)	{ $srcDir     = $opt_s; }
  if (defined $opt_g)	{ $sCfgDir    = $opt_g; }
  if (defined $opt_b)	{ $binDir     = $opt_b; }
  if (defined $opt_c)	{ $configDir  = $opt_c; }
  if (defined $opt_r)	{ $PERL_LOC   = $opt_r; }
  if (defined $opt_f)	{ $optionFile = $opt_f; }
  if (defined $opt_o)	{ $owner      = $opt_o; }
  if (defined $opt_x)
  {
    if ($opt_x eq "n" || $opt_x eq "N" || $opt_x eq "0")
    {
      $overRight = 0;
    }
    else
    {
      $overRight = 1;
    }
  }

  if (defined $opt_u)
  {
    if ($opt_u eq "n" || $opt_u eq "N" || $opt_u eq "0")
    {
      $updateSys = 0;
    }
    else
    {
      $updateSys = 1;
    }
  }

  if (defined $opt_d)	{ $DEBUG = 1; }
  if (defined $opt_h)	{ &usage(); }
}

# --------------------------------------------------------------------------
# Function: setDefaults()
#
# Description:
# --------------------------------------------------------------------------

sub setDefaults
{
  local($sBinDir, *sCfgDir, *binDir, *configDir, *PERL_LOC, *owner,
	*overRight, *updateSys) = @_;

  if ($sCfgDir   eq "") { $sCfgDir   = "$sBinDir"; }
  if ($binDir    eq "") { $binDir    = "/etc/Tivoli/lfa"; }
  if ($configDir eq "") { $configDir = "$binDir"; }
  if ($PERL_LOC  eq "") { $PERL_LOC  = "/usr/bin/perl"; }
  if ($owner     eq "") { $owner     = "root"; }
  if ($overRight eq "") { $overRight = 1; }
  if ($updateSys eq "") { $updateSys = 1; }
}

# --------------------------------------------------------------------------
# Function: convertForwardToBackSlashes()
#
# Description:
# --------------------------------------------------------------------------

sub convertForwardToBackSlashes
{
  local(*path) = @_;

  $path =~ tr:/:\\:;
}

# --------------------------------------------------------------------------
# Function: convertBackToForwardSlashes()
#
# Description:
# --------------------------------------------------------------------------

sub convertBackToForwardSlashes
{
  local(*path) = @_;

  $path =~ tr:\\:/:;
}

# --------------------------------------------------------------------------
# Function: evalSystemReturnCode()
#
# Description:
# --------------------------------------------------------------------------

sub evalSystemReturnCode
{
  local($rc) = @_;

  $rc = 0xffff & $rc;

  if ($rc == 0) { return(0); }

  elsif ($rc == 0xff00) { return(1); }

  elsif ($rc > 0x80)
  {
    $rc >>= 8;
    return($rc);
  }
  else
  {
    if ($rc & 0x80)
    {
      $rc &= ~0x80;
      &bailOnError($rc, "command core dumped: rc = $rc\n");
    }

    &trace("command stopped with signal: $rc\n");
    return($rc);
  }

  return($rc);
}

# --------------------------------------------------------------------------
# Function: getInterpType()
#
# Description: determine interp; modified to only support AIX,
# 	       since we only have to support said platform
#              (we already make the nt service)
#
# --------------------------------------------------------------------------

sub getInterpType
{
  local($line);

  # if ($WINNT) { $interp = "winnt"; }
  if ($WINNT) { &bailOnError(1, "Windows NT is not supported!\n"); }
  else
  {
    open(fh, "uname -s |");
    $interp = <fh>;
    close(fh);

    chop($interp);

    &bailOnError(1, "can't determine interp type\n") if ($interp eq "");

    if    ($interp eq "AIX")   { $interp = "aix"; }
    # elsif ($interp eq "SunOS") { $interp = "sol"; }
    elsif ($interp eq "SunOS") { &bailOnError(1, "Solaris is not supported!\n"); }
    # elsif ($interp eq "HP-UX") { $interp = "hp";  }
    elsif ($interp eq "HP-UX") { &bailOnError(1, "HP/UX is not supported!\n"); }
    else  { &bailOnError(1, "unknown interp type: $interp"); }
  }
}

# --------------------------------------------------------------------------
# Function: validateUser()
#
# Description:
# --------------------------------------------------------------------------

sub validateUser
{
  local($login);

  $login =  getlogin;
  $login =~ tr/A-Z/a-z/;

  if ($WINNT)
  {
    &bailOnError(1, "This program must be run as user: Administrator\n")
	if ($login ne "administrator");
  }
  else
  {
    &bailOnError(1, "This program must be run as user: root\n")
	if ($login ne "root");
  }
}

# --------------------------------------------------------------------------
# Function: validateTargetOwner()
#
# Description:
# --------------------------------------------------------------------------

sub validateTargetOwner
{
  local($user) = @_;
  local($pwd);

  if (! $WINNT)
  {
    @pwd = getpwnam($user);

    &bailOnError(1, "user $user was not found\n") if ($pwd[2] eq "");
  }
}

# --------------------------------------------------------------------------
# Function: verifySrcDir()
#
# Description: Verify that the source dir contains:
#
#	$srcDir/$PROD
#	$srcDir/$PROD.conf
#	$srcDir/$PROD.trace
#	$srcDir/$PROD/rb/classes
#	$srcDir/$PROD/rb/rules
#	$srcDir/$PROD/rb/collectors
# --------------------------------------------------------------------------

sub verifySrcDir
{
  local(*srcDir) = @_;
  local($filename);
  local($dirname);

  &trace("verifying source dirctory: $srcDir\n");

  if ($srcDir eq "")
  {
    print "The source directory is a mandatory option.\n";
    return(1);
  }

  &bailOnError(1, "directory $srcDir does not exist\n")     if ! -d $srcDir;

  #$filename = "$srcDir$sep$PROD";
  #&bailOnError(1, "executable $filename does not exist\n")  if ! -f $filename;

  #$filename = "$srcDir$sep$PROD.conf";
  #&bailOnError(1, "config file $filename does not exist\n") if ! -f $filename;

  #$dirname = "$srcDir$sep$RB_DIR_NAME";
  #&bailOnError(1, "rb directory $dirname does not exist\n") if ! -d $dirname;

  #$dirname = "$srcDir$sep$RB_DIR_NAME$sep$RULES_DIR_NAME";
  #&bailOnError(1, "rules dir $dirname does not exist\n")    if ! -d $dirname;

  #$dirname = "$srcDir$sep$RB_DIR_NAME$sep$CLASS_DIR_NAME";
  #&bailOnError(1, "classes dir $dirname does not exist\n")  if ! -d $dirname;

  &trace("done\n");

  return(0);
}

# --------------------------------------------------------------------------
# Function: chownByName()
#
# Description:
# --------------------------------------------------------------------------

sub chownByName
{
  local($user, $pattern) = @_;
  local($pwd, $grp);

  if (! $WINNT)
  {
    @pwd = getpwnam($user);

    &bailOnError(1, "user $user was not found\n") if ($pwd[2] eq "");

    if (index($pattern, "\*") >= 0)
    {
      system("chown -R $user $pattern");

      @grp = getgrgid($pwd[3]);
      system("chgrp -R $grp[0] $pattern");
    }
    else
    {
      chown($pwd[2], $pwd[3], $pattern);
    }
  }
}

# --------------------------------------------------------------------------
# Function: mkdir()
#
# Description:
# --------------------------------------------------------------------------

sub mkdir
{
  local($dirname, $mod, $owner) = @_;

  if (-d $dirname)
  {
    &trace("dir $dirname already exists\n");

    &bailOnError(1, "no read permissions on $dirname")    if ! -r $dirname;
    &bailOnError(1, "no write permissions on $dirname")   if ! -w $dirname;
    &bailOnError(1, "no execute permissions on $dirname") if ! -x $dirname;

    return;
  }

  # mkdir $dirname, $mod;

  &bailOnError(1, "mkdir failed to create $dirname") if ! -d $dirname;

  &chownByName($owner, $dirname);

  &trace("dir $dirname sucessfully created\n");
}

# --------------------------------------------------------------------------
# Function: mkpath()
#
# Description:
# --------------------------------------------------------------------------

sub mkpath
{
  local($dirname, $mod, $owner) = @_;
  local($paths, $currPath);
  local($i);

  &convertForwardToBackSlashes(*dirname);
  @paths = split(/\\/, $dirname);

  for ($i = 1, $currPath = ""; $i <= $#paths; $i++)
  {
    $currPath = "$currPath$sep@paths[$i]";
    &mkdir($currPath, $mod, $owner);
  }
}

# --------------------------------------------------------------------------
# Function: createTargetDir()
#
# Description:
# --------------------------------------------------------------------------

sub createTargetDir
{
  local($dirname, $dirType, $manditory, $owner, $mod, $overRight) = @_;
  local($msg);

  &trace("creating $dirType dirctory: $dirname\n");

  if ($dirname eq "" && $mandotory == 1)
  {
    print "The $dirType directory is a manditory.\n";
    return(1);
  }

  if (-d $dirname)
  {
    &trace("$dirType dir $dirname already exists\n");

    if ($overRight == 0)
    {
      $msg  = "$dirType directory $dirname already exists,\n";
      $msg .= "rerun install with overRight enabled to continue.\n";

      &bailOnError(1, "$msg");
    }

    return(0);
  }

  &mkpath($dirname, $mod, $owner);

  &trace("done\n");

  return(0);
}

# --------------------------------------------------------------------------
# Function: createDirs()
#
# Description:
# --------------------------------------------------------------------------

sub createDirs
{
  local($binDir, $configDir, $rbDir, $owner, $overRight) = @_;
  local($rc);

  $rc = &createTargetDir($binDir, "executable", 1, $owner, 0755, $overRight);
  &bailOnError($rc, "createTargetDir() failed\n");

  $rc = &createTargetDir($configDir, "config", 0, $owner, 0755, $overRight);
  &bailOnError($rc, "createTargetDir() failed\n");

  $rc = &createTargetDir($rbDir, "class/rules", 0, $owner, 0755, $overRight);
  &bailOnError($rc, "createTargetDir() failed\n");

  $rc = &createTargetDir("$rbDir$sep$CLASS_DIR_NAME", "classes", 0, $owner,
			0755, $overRight);
  &bailOnError($rc, "createTargetDir() failed\n");

  $rc = &createTargetDir("$rbDir$sep$RULES_DIR_NAME", "rules", 0, $owner,
			0755, $overRight);
  &bailOnError($rc, "createTargetDir() failed\n");
}

# --------------------------------------------------------------------------
# Function: copyBinFiles()
#
# Description:
# --------------------------------------------------------------------------

sub copyBinFiles
{
  local($srcDir, $targetDir) = @_;
  local($fileList);
  local($cmd);
  local($i);

  &trace("copy binaries from $srcDir to $targetDir\n");

  umask 0222;

  if ($WINNT)
  {
    $CP = "copy";
    @fileList = ("$PROD.exe", "$SERV_MGR.exe");
  }
  else
  {
    $CP = "cp";
    @fileList = ("$PROD.$interp");
  }

  for ($i = 0; $i <= $#fileList; $i++)
  {
    $cmd = "$CP $srcDir$sep$fileList[$i] $targetDir";
    system($cmd);

    if (! $WINNT)
    {
      $cmd = "chmod 555 $targetDir$sep$fileList[$i]";
      system($cmd);
    }
  }

  if (! $WINNT)
  {
    $cmd = "mv $targetDir$sep$PROD.$interp $targetDir$sep$PROD";
    system($cmd);

    &chownByName($owner, "$targetDir$sep\*");
  }

  &trace("done\n");

  return(0);
}

# --------------------------------------------------------------------------
# Function: copyConfigFiles()
#
# Description:
# --------------------------------------------------------------------------

sub copyConfigFiles
{
  local($srcDir, $targetDir) = @_;
  local($fileList);
  local($cmd);
  local($i);

  &trace("copy configuration files from $srcDir to $targetDir\n");

  if ($WINNT) { $CP = "copy"; }
  else        { $CP = "cp"; }

  @fileList = ("$PROD.conf", "$PROD.trace");

  for ($i = 0; $i <= $#fileList; $i++)
  {
    $cmd = "$CP $srcDir$sep$fileList[$i] $targetDir";
    system($cmd);

    if (! $WINNT)
    {
      $cmd = "chmod 664 $targetDir$sep$fileList[$i]";
      system($cmd);
    }
  }

  if (! $WINNT){ &chownByName($owner, "$targetDir$sep\*"); }

  &trace("done\n");

  return(0);
}

# --------------------------------------------------------------------------
# Function: copyRbFiles()
#
# Description:
# --------------------------------------------------------------------------

sub copyRbFiles
{
  local($srcDir, $targetDir) = @_;
  local($fileList);
  local($cmd);
  local($i);

  &trace("copy class/rule files from $srcDir to $targetDir\n");

  if ($WINNT) { $CP = "copy"; }
  else        { $CP = "cp"; }

  $cmd = "$CP $srcDir$sep$RULES_DIR_NAME$sep\* $targetDir$sep$RULES_DIR_NAME";
  system($cmd);

  $cmd = "$CP $srcDir$sep$CLASS_DIR_NAME$sep\* $targetDir$sep$CLASS_DIR_NAME";
  system($cmd);

  if (! $WINNT)
  {
    $cmd = "chmod -R 664 $targetDir$sep\*";
    system($cmd);
  }

  if (! $WINNT){ &chownByName($owner, "$targetDir$sep\*"); }

  &trace("done\n");

  return(0);
}

# --------------------------------------------------------------------------
# Function: copyFiles()
#
# Description:
# --------------------------------------------------------------------------

sub copyFiles
{
  local($srcDir, $sCfgDir, $binDir, $configDir, $rbDir, $overRight) = @_;
  local($rulesDir);

  &copyBinFiles($srcDir, $binDir);
  &copyConfigFiles($sCfgDir, $configDir);

  $rulesDir = "$sCfgDir$sep$RB_DIR_NAME";
  &copyRbFiles($rulesDir, $rbDir);
}

# --------------------------------------------------------------------------
# Function: addHeader()
#
# Description:
# --------------------------------------------------------------------------

sub addHeader
{
  local($colOne);

  if ($WINNT) { $colOne = "rem"; }
  else        { $colOne = "#"; }

  print envFH "$colOne ---------------------------------------------------\n";
  print envFH "$colOne (C) COPYRIGHT IT Masters, Inc. 1994-1997\n";
  print envFH "$colOne All Rights Reserved\n";
  print envFH "$colOne Licensed Material - Property of IT Masters, Inc.\n";
  print envFH "$colOne\n";
  print envFH "$colOne Component Name: Installation shell Environment\n";
  print envFH "$colOne ---------------------------------------------------\n";
  print envFH "\n";
}

# --------------------------------------------------------------------------
# Function: createNtEtcMcell()
#
# Description:
# --------------------------------------------------------------------------

sub createNtEtcMcell
{
  local($itcHome, $binDir, $overRight) = @_;
  local($envFile);

  $rc = &createTargetDir($ENV_DIR_PATH, "env source", 0, "root", 0755,
		$overRight);
  &bailOnError($rc, "createTargetDir() failed\n");

  # --- create .bat file now

  $envFile = "$ENV_DIR_PATH/env.cmd";

  &trace("create environment source file: $envFile\n");

  &convertForwardToBackSlashes(*envFile);

  open(envFH, "> $envFile") || &bailOnError(1, "Cannot open: $envFile\n");

  print envFH "\@echo off\n";

  &addHeader();

  print envFH "set BINDIR=$binDir\n";
  print envFH "set ITC_HOME=$itcHome\n";
  print envFH "echo done\n";
  print envFH "echo on\n";

  close(enfFH);

  # --- set file perms

  $cmd = "chmod 755 $ENV_DIR_PATH$sep\*";
  system($cmd);

  &trace("done\n");
}

# --------------------------------------------------------------------------
# Function: createUnixEtcMcell()
#
# Description:
# --------------------------------------------------------------------------

sub createUnixEtcMcell
{
  local($itcHome, $binDir, $overRight) = @_;
  local($envFile);

  $rc = &createTargetDir($ENV_DIR_PATH, "env source", 0, "root", 0755,
		$overRight);
  &bailOnError($rc, "createTargetDir() failed\n");

  # --- create .sh file now

  $envFile = "$ENV_DIR_PATH/env.sh";

  &trace("create environment source file: $envFile\n");

  &convertBackToForwardSlashes(*envFile);

  open(envFH, "> $envFile") || &bailOnError(1, "Cannot open: $envFile\n");

  &addHeader();

  print envFH "BINDIR=$binDir\n";
  print envFH "export BINDIR\n";
  print envFH "\n";
  print envFH "PATH=\$PATH:\$BINDIR\n";
  print envFH "export PATH\n";
  print envFH "\n";
  print envFH "ITC_HOME=$itcHome\n";
  print envFH "export ITC_HOME\n";

  close(enfFH);

  # --- create .csh file now

  $envFile = "$ENV_DIR_PATH/env.csh";

  &trace("create environment source file: $envFile\n");

  &convertBackToForwardSlashes(*envFile);

  open(envFH, "> $envFile") || &bailOnError(1, "Cannot open: $envFile\n");

  &addHeader();

  print envFH "if ( ! \$\?BINDIR ) then\n";
  print envFH "  setenv BINDIR $binDir\n";
  print envFH "endif\n";
  print envFH "\n";
  print envFH "setenv PATH \$PATH:\$BINDIR\n";
  print envFH "\n";
  print envFH "if ( ! \$\?ITC_HOME ) then\n";
  print envFH "  setenv ITC_HOME $itcHome\n";
  print envFH "endif\n";

  close(enfFH);

  # --- create $PROD.sh file now

  $envFile = "$ENV_DIR_PATH/$PROD.sh";

  &trace("create server start/stop file: $envFile\n");

  if ($WINNT) { &convertForwardToBackSlashes(*envFile); }
  else        { &convertBackToForwardSlashes(*envFile); }

  open(envFH, "> $envFile") || &bailOnError(1, "Cannot open: $envFile\n");

  print envFH "#!/bin/sh\n";

  &addHeader();

  print envFH "BINDIR=$binDir\n";
  print envFH "export BINDIR\n";
  print envFH "\n";
  print envFH "PATH=\$PATH:\$BINDIR\n";
  print envFH "export PATH\n";
  print envFH "\n";
  print envFH "ITC_HOME=$itcHome\n";
  print envFH "export ITC_HOME\n";
  print envFH "\n";

  print envFH "case \"\$1\" in\n";
  print envFH "  start)\n";
  print envFH "    $PROD -c \$ITC_HOME\n";
  print envFH "    ;;\n";
  print envFH "\n";
  print envFH "  stop)\n";
  print envFH "    pid=`ps -e | grep itcell | awk '{ print \$1 }'`\n";
  print envFH "\n";
  print envFH "    if [ \"x\$pid\" = \"x\" ]; then\n";
  print envFH "      echo \"$PROD server is not currently running\"\n";
  print envFH "    else\n";
  print envFH "      kill \$pid\n";
  print envFH "    fi\n";
  print envFH "\n";
  print envFH "    ;;\n";
  print envFH "\n";
  print envFH "  \*)\n";
  print envFH "    echo \"Unknown flag passed to \$0.\"\n";
  print envFH "    echo \"Usage: \$0 {start, stop}\"\n";
  print envFH "esac\n";

  close(enfFH);

  # --- set file perms

  $cmd = "chmod 755 $ENV_DIR_PATH$sep\*";
  system($cmd);

  &trace("done\n");
}

# --------------------------------------------------------------------------
# Function: registerService()
#
# Description:
# --------------------------------------------------------------------------

sub registerService
{
  local($binDir, $startOption) = @_;
  local($cmd);

  return if (! $WINNT);

  &trace("Registering $PROD as an NT service\n");

  # Unregister first, just in case it is already registered
  # as a service.

  $cmd  = "$binDir$sep$SERV_MGR";
  $cmd .= ".exe -u";

  system("$cmd");

  # Now register it.

  $cmd  = "$binDir$sep$SERV_MGR";
  $cmd .= ".exe -r -e $binDir$sep$PROD";
  $cmd .= ".exe";

  $rc = system("$cmd");
  # $rc = evalSystemReturnCode($rc);

  if ($rc != 0) { print stderr "unable to add $PROD as an NT service\n"; }
}

# --------------------------------------------------------------------------
# Function: updateUnixSysFiles()
#
# Description:
# --------------------------------------------------------------------------

sub updateUnixSysFiles
{
  local($binDir) = @_;

  return if ($WINNT);

  if ($interp eq "aix")
  {
    $rc = &updateAixSysFiles($interp, $PROD, $TRACEON, $binDir, $ENV_DIR_PATH);
  }
  #elsif ($interp eq "sol")
  #{
  #  $rc = &updateSolSysFiles($interp, $PROD, $TRACEON, $binDir, $ENV_DIR_PATH);
  #}
  #elsif ($interp eq "hp")
  #{
  #  $rc = &updateHpSysFiles($interp, $PROD, $TRACEON, $binDir, $ENV_DIR_PATH);
  #}
}

# --------------------------------------------------------------------------
# Function: main()
# --------------------------------------------------------------------------

$PROGRAM = substr($0, 0, index($0, "."));	# strip the extension '.pl'
$TRACEON = 1;

if ($#ARGV < 1) { &usage(); }

&parseCmdLine(*srcDir, *sCfgDir, *binDir, *configDir, *PERL_LOC, *optionFile,
	*owner, *overRight, *updateSys);

&readConfigFile($optionFile, *srcDir, *sCfgDir, *binDir, *configDir,
		*PERL_LOC, *owner, *overRight, *updateSys);

#&setDefaults($srcDir, *sCfgDir, *binDir, *configDir, *PERL_LOC, *owner,
#		*overRight, *updateSys);

if ($srcDir eq "" && $optionFile eq "")
  { &bailOnError(1, "Either the -s or -f flags are mandatory\n"); }

if ($ENV{'OS'} eq "Windows_NT")
{
  $WINNT = 1;
  $sep = "\\";
}
else
{
  $WINNT = 0;
  $sep = "/";
}

&getInterpType();

#print "test copy if itcell.$interp\n\n";
#print "validateUser() disabled - ENABLE LATER\n\n";

  &validateUser();
  &validateTargetOwner($owner);

  if ($WINNT)
  {
    &convertForwardToBackSlashes(*srcDir);
    &convertForwardToBackSlashes(*binDir);
    &convertForwardToBackSlashes(*configDir);
    &convertForwardToBackSlashes(*PERL_LOC);
  }
  else
  {
    &convertBackToForwardSlashes(*srcDir);
    &convertBackToForwardSlashes(*binDir);
    &convertBackToForwardSlashes(*configDir);
    &convertBackToForwardSlashes(*PERL_LOC);
  }

&trace("optionFile: $optionFile\n");
&trace("srcBinDir : $srcDir\n");
&trace("srcCfgDir : $sCfgDir\n");
&trace("binDir    : $binDir\n");
&trace("configDir : $configDir\n");
&trace("PERL_LOC : $PERL_LOC\n");
&trace("owner     : $owner\n");
&trace("overRight : $overRight\n");
&trace("updateSys : $updateSys\n");
&trace("DEBUG     : $DEBUG\n");
&trace("WINNT     : $WINNT\n");
&trace("INTERP    : $interp\n");

#
# make sure the source directory looks intact before proceeding
#

$rc = &verifySrcDir(*srcDir);
&bailOnError($rc, "verifySrcDir() failed\n");

#
# Create the target directory to install into
#

# &createDirs($binDir, $configDir, $rbDir, $owner, $overRight);

#
# Copy the files into the approapriate directories
#

# &copyFiles($srcDir, $sCfgDir, $binDir, $configDir, $rbDir, $overRight);

#
# Create the /etc/$PROD/env.[c]sh directory and files
#

#if ($WINNT)
#  { &createNtEtcMcell($configDir, $binDir, $overRight); }
#else
#  { &createUnixEtcMcell($configDir, $binDir, $overRight); }

#
# If this is being installed on NT, then register the server
# as a service. If on Unix, then update system files.
#

#if ($WINNT)
  #{ &registerService($binDir, $startOption) if ($updateSys == 1); }
#else
 # { &updateUnixSysFiles($binDir) if ($updateSys == 1); }

&updateUnixSysFiles($binDir, $PERL_LOC) if ($updateSys == 1);
&trace("Install has completed sucessfully\n");
